@@ -399,6 +399,24 @@ print.errorValue <- function(x, ...) {
 
 }
 
+#' @export
+#'
+print.conditionVariable <- function(x, ...) {
+
+  cat("< conditionVariable >\n", file = stdout())
+  invisible(x)
+
+}
+
+#' @export
+#'
+print.nanoPipe <- function(x, ...) {
+
+  cat(sprintf("< nanoPipe >\n - id: %d\n", attr(x, "id")), file = stdout())
+  invisible(x)
+
+}
+
 #' @export
 #'
 `[[.nano` <- function(x, i, exact = FALSE)
