@@ -46,7 +46,7 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP raw) {
 
   nng_socket *sock;
   const char *pname;
-  int xc = 0;
+  int xc;
   SEXP socket, klass;
 
   sock = R_Calloc(1, nng_socket);
@@ -119,12 +119,14 @@ SEXP rnng_protocol_open(SEXP protocol, SEXP raw) {
       break;
     }
   default:
+    R_Free(sock);
     Rf_error("'protocol' should be one of bus, pair, push, pull, pub, sub, req, rep, surveyor, respondent");
+    xc = 0;
   }
 
   if (xc) {
     R_Free(sock);
-    return mk_error(xc);
+    ERROR_OUT(xc);
   }
 
   PROTECT(socket = R_MakeExternalPtr(sock, nano_SocketSymbol, R_NilValue));
@@ -149,9 +151,8 @@ SEXP rnng_close(SEXP socket) {
     Rf_error("'socket' is not a valid Socket");
   nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
   const int xc = nng_close(*sock);
-
   if (xc)
-    return mk_error(xc);
+    ERROR_RET(xc);
 
   Rf_setAttrib(socket, nano_StateSymbol, Rf_mkString("closed"));
   return nano_success;
