@@ -5,6 +5,8 @@ nanotestz <- function(x) invisible(x == 0L || stop())
 nanotestnano <- function(x) invisible(is_nano(x) || stop())
 nanotestn <- function(x) invisible(is.null(x) || stop())
 nanotestnn <- function(x) invisible(!is.null(x) || stop())
+nanotestp <- function(x) invisible(is.character(capture.output(print(x))) || stop())
+nanotestxp <- function(x) invisible(typeof(x) == "externalptr" || stop())
 nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop())
 
@@ -13,7 +15,7 @@ n1 <- nano("pair", dial = "inproc://nanonext", autostart = FALSE)
 nanotest(inherits(n, "nanoObject"))
 nanotest(inherits(n$socket, "nanoSocket"))
 nanotest(inherits(n$socket, "nano"))
-nanotest(is_nano(n))
+nanotestnano(n)
 n$newmethod <- "doesnotwork"
 nanotestn(n$newmethod)
 nanotest(is.integer(attr(n$socket, "id")))
@@ -27,7 +29,7 @@ nanotestnano(n$opt("recv-buffer", 8L))
 nanotestnano(n$opt("socket-name", "nano"))
 nanotest(n$opt("socket-name") == "nano")
 nanotesterr(n$opt("socket-name", NULL), "argument")
-n$listener[[1]]
+nanotestp(n$listener[[1]])
 nanotest(inherits(n$listener[[1]], "nanoListener"))
 nanotest(n$listener[[1]]$url == "inproc://nanonext")
 nanotest(n$listener[[1]]$state == "not started")
@@ -41,7 +43,7 @@ nanotesterr(n$listener_opt("false", TRUE), "supported")
 nanotesterr(n$listener_opt("false", list()), "type")
 nanotestz(n$listener_start())
 nanotest(n$listener[[1]]$state == "started")
-n1$dialer[[1]]
+nanotestp(n1$dialer[[1]])
 nanotest(inherits(n1$dialer[[1]], "nanoDialer"))
 nanotest(n1$dialer[[1]]$url == "inproc://nanonext")
 nanotest(n1$dialer[[1]]$state == "not started")
@@ -62,9 +64,10 @@ nanotesterr(n$send(list(), mode = "raw"), "atomic vector type")
 nanotesterr(n$recv(mode = "none"), "mode")
 nanotesterr(n$recv(mode = "c"), "mode")
 raio <- n$recv_aio(timeout = 1L)
-raio
+nanotestp(raio)
 nanotest(is_error_value(call_aio(raio)$data))
 nanotest(is_error_value(raio$data))
+nanotesterr(msg_pipe(raio), "successful Aio")
 nanotestz(n$send(data.frame(), block = FALSE))
 nanotest(is.list(n1$recv(block = FALSE, keep.raw = TRUE)))
 nanotestz(n1$send("test", mode = "raw", block = 500))
@@ -72,13 +75,13 @@ nanotest(n$recv("character", block = 500, keep.raw = TRUE)$data == "test")
 nanotestz(n1$send(c(1L, 0L, 0L), mode = "r"))
 nanotest(is.list(n$recv("int", keep.raw = TRUE)))
 saio <- n$send_aio(data.frame(), timeout = 500)
-saio
+nanotestp(saio)
 nanotest(is_aio(call_aio(saio)))
 nanotest(!is_nano(saio))
 nanotest(is.integer(saio$result))
 nanotesterr(n$send("wrong mode", mode = "none"), "mode")
 raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE)
-raio
+nanotestp(raio)
 raio$newfield <- "doesnotwork"
 nanotestn(raio$newfield)
 nanotest(is_aio(raio))
@@ -86,6 +89,7 @@ saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500)
 saio$newfield <- "doesnotwork"
 nanotestn(saio$newfield)
 nanotest(is_aio(saio))
+nanotest(is.logical(.unresolved(saio)))
 nanotest(is.logical(unresolved(saio)))
 nanotestnn(raio$raw)
 nanotest(identical(unserialize(call_aio(raio)$raw), raio$data))
@@ -95,11 +99,16 @@ nanotestn(stop_aio(n))
 nanotest(identical(call_aio(msg), msg))
 nanotesterr(msg$raw, "active")
 nanotesterr(msg$data, "active")
+nanotesterr(msg_pipe(msg), "active")
 nanotest(identical(call_aio(n), n))
 nanotest(!is_aio(n))
 sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500)
 rraio <- n1$recv_aio(mode = "r", timeout = 500)
 nanotestnn(call_aio(rraio)$data)
+nanotestxp(p <- msg_pipe(rraio))
+nanotest(is.integer(attr(p, "id")))
+nanotestp(p)
+nanotestz(close(p))
 sraio <- n$send_aio(c(1+2i, 4+3i), mode = "ra", timeout = 500)
 rraio <- n1$recv_aio(mode = "co", timeout = 500)
 nanotestnn(call_aio(rraio)$data)
@@ -156,14 +165,14 @@ nanotest(n1$socket[state] == "closed")
 
 req <- nano("req", listen = "inproc://testing")
 rep <- socket("rep", dial = "inproc://testing", listen = "inproc://testing2")
-rep
+nanotestp(rep)
 nanotest(stat(rep, "dialers") == 1)
 nanotest(stat(rep, "protocol") == "rep")
 nanotestn(stat(rep, "nonexistentstat"))
 nanotestnano(req$opt("req:resend-time", 1000))
 nanotest(req$opt("req:resend-time") == 1000L)
 nanotesterr(req$opt("none"), "supported")
-nanotestnn(req$context_open())
+nanotestxp(req$context_open())
 nanotest(inherits(req$context, "nanoContext"))
 nanotest(inherits(req$context, "nano"))
 nanotest(is.integer(req$context$id))
@@ -178,7 +187,7 @@ nanotesterr(req$opt("false", NULL), "supported")
 nanotesterr(req$opt("false", TRUE), "supported")
 nanotesterr(req$opt("false", list()), "type")
 ctx <- context(rep)
-ctx
+nanotestp(ctx)
 csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500)
 nanotestnn(call_aio(csaio)$result)
 nanotestnn(recv_aio(ctx, timeout = 500))
@@ -186,6 +195,7 @@ nanotestnn(req$send("context test", mode ="raw", block = FALSE))
 nanotestnn(recv(ctx, mode = "ch", block = FALSE))
 nanotestnn(req$send(data.frame(), mode = "seri", block = 500))
 msg <- recv_aio(ctx, mode = "ser", timeout = 500, keep.raw = FALSE)
+nanotest(is.logical(.unresolved(msg)))
 nanotest(is.logical(unresolved(msg)))
 nanotestnn(call_aio(msg)$data)
 nanotest(!unresolved(msg))
@@ -194,23 +204,48 @@ msg <- recv_aio(ctx, mode = 6L, timeout = 500, keep.raw = TRUE)
 nanotestnn(call_aio(msg)$raw)
 nanotestnn(msg$raw)
 err <- send_aio(ctx, "test")
-stop_aio(err)
+nanotestn(stop_aio(err))
 nanotesterr(err$result, "active")
 err <- send_aio(ctx, "test")
 nanotest(is_error_value(call_aio(err)$result))
 nanotestnn(request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500, keep.raw = TRUE))
 nanotestnn(reply(ctx, execute = identity, send_mode = "ra", timeout = 500))
+
+nanotest(inherits(cv <- cv(), "conditionVariable"))
+nanotestp(cv)
+nanotest(until(cv, 10L))
+nanotest(until(cv, 10))
+nanotestn(cv_reset(cv))
+nanotestz(cv_value(cv))
+nanotestnn(cr <- request_signal(req$context, "test", timeout = 500, keep.raw = TRUE, cv = cv))
+nanotestnn(cr <- recv_aio_signal(rep, timeout = 500, keep.raw = TRUE, cv = cv))
+nanotestnn(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
+nanotestz(pipe_notify(rep, cv))
+nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
+nanotesterr(request_signal(err, "test", cv = cv), "valid")
+nanotesterr(recv_aio_signal(err, timeout = 500, cv = cv))
+nanotesterr(request_signal(req$context, "test", cv = err), "valid")
+nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
+nanotesterr(wait(err), "valid")
+nanotesterr(until(err, 10), "valid")
+nanotesterr(cv_value(err), "valid")
+nanotesterr(cv_reset(err), "valid")
+nanotesterr(pipe_notify(err, cv), "valid Socket")
+nanotesterr(pipe_notify(rep, err), "valid Condition Variable")
 nanotestz(req$context_close())
 nanotestn(req$context_close)
 nanotestz(req$close())
 nanotestn(req$context)
 rep$dialer <- NULL
-nanotestnn(rep$dialer)
+nanotestxp(rep$dialer[[1L]])
 nanotestz(close(ctx))
 nanotestz(close(rep))
 
 pub <- nano("pub", listen = "inproc://ps")
 sub <- nano("sub", dial = "inproc://ps", autostart = NA)
+cv <- cv()
+cv2 <- cv()
+nanotestz(pipe_notify(pub$socket, cv, cv2, flag = FALSE))
 nanotestnano(sub$opt(name = "sub:prefnew", value = FALSE))
 nanotest(!sub$opt(name = "sub:prefnew"))
 nanotesterr(sub$opt(name = "false", value = 100), "supported")
@@ -219,7 +254,7 @@ nanotesterr(sub$opt(name = "false", value = list()), "type")
 nanotestnano(sub$subscribe("test"))
 nanotestnano(subscribe(sub$socket, NULL))
 nanotestnano(sub$unsubscribe("test"))
-nanotestnn(sub$context_open())
+nanotestxp(sub$context_open())
 nanotest(inherits(sub$context, "nanoContext"))
 nanotestnano(sub$subscribe(12))
 nanotestnano(sub$unsubscribe(12))
@@ -228,21 +263,26 @@ nanotestz(sub$context_close())
 nanotestn(sub$context)
 nanotestz(sub$close())
 nanotestz(pub$close())
+nanotest(wait(cv))
+nanotest(wait(cv2))
 
 surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2")
-surv
+nanotestp(surv)
 resp <- nano(protocol = "respondent", listen = "inproc://sock2", dial = "inproc://sock1")
+nanotestz(pipe_notify(surv$socket, cv, cv2, flag = TRUE))
 surv$dialer <- NULL
-nanotestnn(surv$dialer)
-nanotestnn(surv$listener)
+nanotestxp(surv$dialer[[1L]])
+nanotestxp(surv$listener[[1L]])
 nanotestnano(surv$survey_time(5000))
-nanotestnn(surv$context_open())
-nanotestnn(resp$context_open())
+nanotestxp(surv$context_open())
+nanotestxp(resp$context_open())
 nanotestnano(surv$survey_time(value = 2000))
 nanotestz(surv$context_close())
 nanotestz(resp$context_close())
 nanotestz(surv$close())
 nanotestz(resp$close())
+nanotest(!wait(cv))
+nanotest(!wait(cv2))
 nanotest(is_error_value(resp$recv(keep.raw = TRUE)$raw))
 
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
@@ -250,9 +290,10 @@ nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
 nanotest(inherits(pull <- socket(protocol = "pull"), "nanoSocket"))
 nanotest(inherits(survey <- nano(protocol = "surveyor"), "nanoObject"))
 nanotest(inherits(respond <- nano(protocol = "respondent"), "nanoObject"))
-nanotest(is_nano(bus))
+nanotestnano(bus)
 nanotestw(listen(bus, url = "test") == 3L)
 nanotestw(dial(bus, url = "test") == 3L)
+nanotestxp(weakref(push) <- pull)
 nanotestz(close(bus))
 nanotestz(close(push))
 nanotestz(close(pull))
@@ -276,23 +317,23 @@ nanotestnn(put1$headers)
 nanotestnn(put1$raw)
 nanotestnn(put1$data)
 put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"))
-put2
+nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
 nanotest(is_error_value(ncurl("http")$raw))
 nanotest(is_error_value(ncurl("http", async = TRUE)$data))
 haio <- ncurl("https://i.i", async = TRUE)
 nanotest(is_error_value(call_aio(haio)$raw))
-haio$data
-sess <- tryCatch(ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server"), error = identity)
+nanotestp(haio$data)
+sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 1000L)
 nanotestnn(sess)
-nanotest(if (!inherits(sess, "error")) length(transact(sess)) == 4L else TRUE)
-sess <- tryCatch(ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server")), error = identity)
+nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
+sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 1000)
 nanotestnn(sess)
-nanotest(if (!inherits(sess, "error")) length(transact(sess)) == 4L else TRUE)
-nanotest(if (!inherits(sess, "error")) close(sess) == 0L else TRUE)
+nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
+nanotest(is_error_value(sess) || close(sess) == 0L)
 nanotesterr(transact(sess), "ncurlSession")
-nanotesterr(ncurl_session("https://i.i"))
+nanotest(is_error_value(ncurl_session("https://i")))
 nanotesterr(stream(dial = "wss://127.0.0.1:5555"))
 nanotesterr(stream(listen = "errorValue3"), "argument")
 nanotesterr(stream(), "specify")
@@ -333,12 +374,15 @@ if (length(s)) {
   nanotest(is.integer(call_aio(ss)[["result"]]))
   nanotestn(stop_aio(ss))
   nanotest(is.integer(send(s, 12.56, mode = "raw", block = 500L)))
-  nanotest(is_aio(sr <- recv_aio(s, mode = "double", timeout = 500L)))
+  nanotest(is_aio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv)))
   nanotestnn(call_aio(sr)[["data"]])
+  nanotest(cv_value(cv) > 0L)
+  nanotest(!wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
   nanotesterr(recv(s, mode = "none", block = FALSE), "mode")
   nanotesterr(recv(s, mode = "c", block = FALSE), "mode")
   nanotesterr(opt(s, "none"), "supported")
+  nanotestp(s)
   nanotest(is.integer(close(s)))
 }
 
@@ -350,14 +394,15 @@ nanotesterr(listen(fakesock, autostart = FALSE), "valid Socket")
 nanotesterr(context(fakesock), "valid Socket")
 nanotesterr(stat(fakesock, "pipes"), "valid Socket")
 nanotesterr(close(fakesock), "valid Socket")
+nanotest(.unresolved(fakesock) == FALSE)
 fakectx <- `class<-`("test", "nanoContext")
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
 nanotesterr(close(fakectx), "valid Context")
 fakestream <- `class<-`("test", "nanoStream")
-fakestream
+nanotestp(fakestream)
 fakesession <- `class<-`("test", "ncurlSession")
-fakesession
+nanotestp(fakesession)
 nanotesterr(transact(fakesession), "valid")
 nanotesterr(close(fakesession), "valid")
 nanotesterr(send(fakestream, "test"), "valid")
@@ -374,10 +419,15 @@ fakelist <- `class<-`("test", "nanoListener")
 nanotesterr(start(fakelist), "valid Listener")
 nanotesterr(close(fakelist), "valid Listener")
 unres <- `class<-`(NA, "unresolvedValue")
-unres
+nanotestp(unres)
 nanotest(unresolved(unres))
 nanotest(identical(call_aio("a"), "a"))
 nanotestn(stop_aio("a"))
+nanotesterr(msg_pipe(fakesock), "valid recvAio")
+fakencurl <- `class<-`("test", "ncurlAio")
+nanotesterr(msg_pipe(fakencurl), "valid recvAio")
+fakepipe <- `class<-`("test", "nanoPipe")
+nanotesterr(close(fakepipe), "valid Pipe")
 
 nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
 nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
@@ -387,6 +437,8 @@ nanotest(length(sha384(data.frame(), "KEY", convert = FALSE)) == 48L)
 nanotest(nchar(sha384(7000L, key = NULL, convert = TRUE)) == 96L)
 nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 28L)
 nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
+nanotest(length(sha1("testingnoncrypto", convert = FALSE)) == 20L)
+nanotest(nchar(sha1(c(1, 2), key = "secret")) == 40L)
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
